 /*
    * Copyright (c) 2012, H. J. Ohlbach. All rights reserved.
    * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
    *
    * This code is free software; you can redistribute it and/or modify it
    * under the terms of the GNU General Public License version 2 only, as
    * published by the Free Software Foundation. 
    *
    * This code is distributed in the hope that it will be useful, but WITHOUT
    * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    * version 2 for more details.
    */
package Java2OWL;

import java.io.FileInputStream;
import java.io.IOException;

/** This class loader loads a class from a file which is given by a pathname. 
 *  It is independent of the CLASSPATH setting. 
 *  It is invoked only if the normal class loading does not find the file.  
 *
 * @author ohlbach
 */
public class FileClassLoader extends ClassLoader {
    
    /** loads a class from the file with the given pathname.
     * 
     * @param file The pathname where the class can be loaded from.  
     * @return the loaded class.
     * @throws ClassNotFoundException if something goes wrong.  
     */
    @Override
    public Class findClass(String file) throws ClassNotFoundException {
      FileInputStream fis = null;
      try {
         fis = new FileInputStream(file);
         byte[] bytes = new byte[fis.available()];
         int read = fis.read(bytes);
         if (read != bytes.length) {throw new ClassNotFoundException("IO-error at file " + file);}
         return defineClass(null, bytes, 0, bytes.length);}
      catch(IOException ex) {throw new ClassNotFoundException("IO-error at file " + file,ex);}
      finally {try{if(fis != null) fis.close();} catch(IOException ex) {}}
   }
}
