/*
    * Copyright (c) 2012, H. J. Ohlbach. All rights reserved.
    * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
    *
    * This code is free software; you can redistribute it and/or modify it
    * under the terms of the GNU General Public License version 2 only, as
    * published by the Free Software Foundation. 
    *
    * This code is distributed in the hope that it will be useful, but WITHOUT
    * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    * version 2 for more details.
    */

package Java2OWL;

import java.lang.reflect.Method;

/** This is the common superclass of PropertyWrapperMappingFunctional and PropertyWrapperMappingRelational
 *
 * @author ohlbach
 */
abstract class PropertyWrapperMapping extends PropertyWrapper {

    /** delegates the construction to the PropertyWrapper class */
    PropertyWrapperMapping(Class jcl, Method getter, J2OManager manager) throws J2OException {
        super(jcl,getter,manager);}
    
    /** finds adder and clearer methods and checks the other ones */
    protected void findAccessors(J2OWLProperty annotation, StringBuffer errors, StringBuffer warnings) {
        currentPWC.setter  = findAccessor("setter", annotation.setter(), false, errors, warnings);
        currentPWC.adder   = findAccessor("adder", annotation.adder(), true, errors, warnings);
        currentPWC.remover = findAccessor("remover", annotation.remover(), false, errors, warnings);
        currentPWC.clearer = findAccessor("clearer", annotation.clearer(), true, errors, warnings);
        currentPWC.getKey  = findAccessor("getKey", annotation.getKey(), currentPWC.keyClass != String.class, errors, warnings);}
     
     
    /** checks the getKey accessor. It must accept a string as argument */
    protected Method checkGetKey(Method accessor, StringBuffer errors) {
        Class[] parameters = accessor.getParameterTypes();
        if(parameters.length != 1 || parameters[0] != String.class) {
            errors.append("The getKey Method has not a single String argument.\n");
            return null;}
        return accessor;}
}
