/*
    * Copyright (c) 2012, H. J. Ohlbach. All rights reserved.
    * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
    *
    * This code is free software; you can redistribute it and/or modify it
    * under the terms of the GNU General Public License version 2 only, as
    * published by the Free Software Foundation. 
    *
    * This code is distributed in the hope that it will be useful, but WITHOUT
    * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    * version 2 for more details.
    */

package Java2OWL;

import java.lang.reflect.Array;
import java.lang.reflect.GenericArrayType;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.logging.Level;
import java.util.logging.Logger;

/** for accessor groups of Map-type, for example HashMap<Locale,List<String>> or HashMap<Locale,String[]>
 * <pre> 
    HashMap&lt;String,Set&lt;TestMapper&gt;&gt; friends = new HashMap&lt;String,Set&ltTestMapper&gt;&gt;();
    
    &#64;J2OWLProperty(name = "hasFriends", setter = "setFriends", adder="addFriends,addFriend", remover="removeFriends,removeFriend", 
            clearer="clearFriends")
    public HashMap&lt;String,Set&lt;TestMapper&gt;&gt; getFriends() {return friends;}
    
    public void setFriends(Object friends) {
        assert(friends instanceof HashMap);        
        this.friends = (HashMap&lt;String,Set&lt;TestMapper&gt;&gt;)friends;}
    
    public Set&lt;TestMapper&gt; getFriends(String key) {return friends.get(key);}
    
    public void addFriends(String key, Object f1) {
        assert(f1 instanceof Set);
        friends.put(key,(Set&lt;TestMapper&gt;)f1);}
    
  
    public void addFriend(String key, TestMapper f) {
        if(friends.get(key) == null) {friends.put(key,new HashSet&lt;TestMapper&gt;());}
        friends.get(key).add(f);
    }
    
    
    public void removeFriends(String key) {friends.remove(key);}
    
    
    public void removeFriend(String key,TestMapper f) {
       Object fr = friends.get(key);
       if(fr != null) {
           ((Set&lt;TestMapper&gt;)fr).remove(f);
           if(((Set&lt;TestMapper&gt;)fr).isEmpty()) friends.remove(key);}}
    
    public void clearFriends() {friends.clear();}
 </pre>
 *
 * @author ohlbach
 */
public class PropertyWrapperMappingRelational extends PropertyWrapperMapping {
    
    PropertyWrapperMappingRelational(Class jcl, Method getter, J2OManager manager) throws J2OException {
        super(jcl,getter,manager);
    }
    
     /** determines the key and range classes */
    protected boolean determineClasses(StringBuffer errors) {
        currentPWC.isFunctional = false;
        currentPWC.isMapper = true;
        currentPWC.exactLength.put("setter",true); // exactly one argument
        currentPWC.exactLength.put("adder",false);
        currentPWC.exactLength.put("remover",false);
        currentPWC.exactLength.put("clearer",true); //no argument
        currentPWC.exactLength.put("getKey",true);  // exactly one argument
        Class rangeClass = currentPWC.getter.getReturnType();
        ParameterizedType rangeType = (ParameterizedType)(currentPWC.getter.getGenericReturnType());
        Type[] componentTypes = rangeType.getActualTypeArguments();
        if(componentTypes.length != 2) {errors.append("Illegal Range Type " + rangeType.toString()); return false;}
        Class keyClass = Utilities.isClass(componentTypes[0]);
        if(keyClass == null) {errors.append("Illegal Range Class " + rangeType); return false;}
        currentPWC.keyClass = keyClass;
        Type collectionType = componentTypes[1];
        Type componentType;
        Class collectionClass;
        if(collectionType instanceof GenericArrayType) {
            GenericArrayType arrayType = (GenericArrayType)collectionType;
            componentType = arrayType.getGenericComponentType();
            collectionClass = Utilities.getArrayClass(arrayType);}
        
        else {
            if(Utilities.isCollection(collectionType) == null)
                {errors.append("No collection Class " + collectionType.toString()); return false;}
            componentTypes = ((ParameterizedType)collectionType).getActualTypeArguments();
            if(componentTypes.length != 1) {errors.append("Illegal Collection Type " + rangeType.toString()); return false;}
            componentType = componentTypes[0];
            collectionClass = (Class)((ParameterizedType)collectionType).getRawType();}
        
        if(collectionClass == null) {
            errors.append("Illegal Range Class " + componentTypes.toString()); return false;}
        Class componentClass = Utilities.isClass(componentType);
        if(componentClass == null) {errors.append("Illegal Range Class " + componentTypes[1].toString()); return false;}
        currentPWC.rangeClass = componentClass;
        currentPWC.types.put("setter",new Class[][]{new Class[]{rangeClass}});
        Class[][] argsadder = new Class[][]{new Class[]{keyClass,componentClass},
                                            new Class[]{keyClass,collectionClass}};
        
        Class[][] argsremover = new Class[][]{new Class[]{keyClass,componentClass},
                                              new Class[]{keyClass}};
        currentPWC.types.put("adder",argsadder);
        currentPWC.types.put("remover",argsremover);
        currentPWC.types.put("clearer",new Class[][]{new Class[]{}});
        currentPWC.types.put("getKey",new Class[][]{new Class[]{keyClass}});
        return true;}
    
    
}
